<div xmlns:py="http://genshi.edgewall.org/"
     class="tagging_tags_cloud ${css_class}"
     id="${id}">
    <ul class="tagging_tags">
        <li py:for="tag in value">
            <a class="tagging_tag tagging_tag_${(6 * (tag[1] - weight_min)) / (weight_range)}"
               title="${tag[0]}"
               href="${tg_url('%s/search' % tagging_url, params=dict(tags=tag[0]))}">
                ${Markup(tag[0])}
            </a>
        </li>
    </ul>
</div>