.tagging_tags {
    padding:0;
}

.tagging_tags li {
    list-style-type: none;
    display: inline;
    margin-right: 5px;
}

.tagging_tags li a {
    text-decoration: none;
}

.tagging_tags li a:hover {
    text-decoration: underline;
}

.tagging_tag_1 {
    font-size: 100%;
}

.tagging_tag_2 {
    font-size: 115%;
}

.tagging_tag_3 {
    font-size: 130%;
}

.tagging_tag_4 {
    font-size: 145%;
}

.tagging_tag_5 {
    font-size: 160%;
}

.tagging_tag_6 {
    font-size: 175%;
}

.tagging_tag_delete {
    background-image: url('delete.png');
    display: inline-block;
    width: 16px;
    height: 16px;
}
